package com.oferteo.api.v1.client.rest;

import java.text.DateFormat;
import java.text.Format;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ApiClientUtils 
{
	public static final String HTTP = "http";
	public static final String HTTPS = "https";

	private static final Pattern PATTERN_PAGER_DIGITS 	= Pattern.compile("^([0-9]+)$");
	private static final Pattern PATTERN_SORT 			= Pattern.compile("^(.*)sort=([0-9-]+)(.*)$");
	private static final Pattern PATTERN_SHOW_DETAILS	= Pattern.compile("^(.*)/([a-zA-Z0-9-]+)-([0-9-]+)$");
	private static final String	DEFAULT_PARAM_URL_NAME = "/";
	
	public static final Map<String, String> CAPITALIZE_EXCLUDED_WORDS = new ConcurrentHashMap<String, String>();
	static
	{
		CAPITALIZE_EXCLUDED_WORDS.put("RTV", "RTV");
		CAPITALIZE_EXCLUDED_WORDS.put("AGD", "AGD");
		CAPITALIZE_EXCLUDED_WORDS.put("AC",  "AC");
		CAPITALIZE_EXCLUDED_WORDS.put("OC",  "OC");
		CAPITALIZE_EXCLUDED_WORDS.put("WWW", "WWW");
		CAPITALIZE_EXCLUDED_WORDS.put("IT",  "IT");
		CAPITALIZE_EXCLUDED_WORDS.put("ERP",  "ERP");
		CAPITALIZE_EXCLUDED_WORDS.put("CMS",  "CMS");
		CAPITALIZE_EXCLUDED_WORDS.put("DVD",  "DVD");
		CAPITALIZE_EXCLUDED_WORDS.put("DO", "DO");
		CAPITALIZE_EXCLUDED_WORDS.put("NA", "NA");
		CAPITALIZE_EXCLUDED_WORDS.put("G-K", "G-K");
		CAPITALIZE_EXCLUDED_WORDS.put("GSM", "GSM");
		CAPITALIZE_EXCLUDED_WORDS.put("UE", "UE");
		CAPITALIZE_EXCLUDED_WORDS.put("BHP", "BHP");
		CAPITALIZE_EXCLUDED_WORDS.put("BIK", "BIK");
		CAPITALIZE_EXCLUDED_WORDS.put("CAD/CAM/CAE", "CAD/CAM/CAE");
		CAPITALIZE_EXCLUDED_WORDS.put("CO", "CO");
		CAPITALIZE_EXCLUDED_WORDS.put("JCB", "JCB");
	};
	
	/**
	 * converts timestamp to String using given format
	 * @param timestamp
	 * @param format
	 * @return
	 */
	public static String getDateStr(long timestamp, Format format) {
		if (timestamp > 0 && format != null) {
			return format.format(new Date(timestamp));
		} else {
			return "";
		}
	}
	
	
	public static String getDateStr(long timestamp) {
		return getDateStr(timestamp, getDateFormat());
	}
	
	public static String getDateTimeStr(long timestamp) {
		return getDateStr(timestamp, getDateTimeFormat());
	}

	/**
	 * @return formatter: yyyy-MM-dd
	 */
	public static DateFormat getDateFormat() {
		return new SimpleDateFormat("yyyy-MM-dd");
	}
	
	/**
	 * @return formatter: yyyy-MM-dd HH:mm:ss
	 */
	public static DateFormat getDateTimeFormat() {
		return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); 
	}
	

	public static String getMessage(Locale locale, String key) {
		return getMessage(locale, key, (Object[])null);
	}
	
    public static String getMessage(Locale locale, String key, Object... params  ) {
        try {
    		ResourceBundle rb = ResourceBundle.getBundle("OferteoApi", locale);
            return MessageFormat.format(rb.getString(key), params);
        } catch (Exception e) {
            return "";
        }
    }

	public static String trimText(String text, String trimStartPhrase, String trimEndPhrase)
	{
		if (text != null) {
			if (trimStartPhrase != null && text.startsWith(trimStartPhrase)) {
				text = text.substring(trimStartPhrase.length());
			}
			if (trimEndPhrase != null && text.endsWith(trimEndPhrase)) {
				if (text.lastIndexOf(trimEndPhrase) >= 0) {
					text = text.substring(0, text.lastIndexOf(trimEndPhrase));
				}
			}
		}
		return text;
	}
    
	public static String trimUrl(String url, String trimStartPhrase, String trimEndPhrase)
	{
		return trimText(url, trimStartPhrase, trimEndPhrase);
	}
    
	public static int findPagerInUrl(String requestedUrl)
	{
		return findLastDigitInUrl(requestedUrl, -1);
	}

	public static int findLastDigitInUrl(String requestedUrl, int limit)
	{
		int pager = -1;
		
		int idxParams = requestedUrl.indexOf("?");
		if (idxParams > 0) {
			requestedUrl = requestedUrl.substring(0, idxParams);
		}
		requestedUrl = trimUrl(requestedUrl, "/", "");
		
		String[] group = requestedUrl.split("/");
		String urlLastPart = group[group.length - 1];
		
		Matcher match = PATTERN_PAGER_DIGITS.matcher(urlLastPart);
		if (match.find()) {	// pager w postaci '1', '2', '10', '550'
			String pagerTmp = match.group(1);
			
			if (Integer.parseInt(pagerTmp) <= limit || limit == -1) {
				pager = Integer.parseInt(pagerTmp);
			}
		}
		
		return pager;
	}

	public static String findCategoryInUrl(String url, String urlPrefix, Map<String, Integer> categoryMap) {
		String category = null;
		String requestedUrl = trimUrl(url, urlPrefix, "/");
		requestedUrl = trimUrl(requestedUrl, "/", "");
		String[] splitted = requestedUrl.split("/");
		
		if (splitted.length > 0) {
			String firstElement = "/" + splitted[0];
			//kategoria moze byc podana tylko na pierwszym miejscu
			if (categoryMap.containsKey(firstElement)) {
				category = firstElement;
			}
		}
		return category;
	}

	public static String findLocationInUrl(String url, String urlPrefix, Map<String, String> locationMap) {
		String location = null;
		String requestedUrl = trimUrl(url, urlPrefix, "/");
		requestedUrl = trimUrl(requestedUrl, "/", "");
		String[] splitted = requestedUrl.split("/");
		
		if (splitted.length > 0) {
			//lokalizacja moze byc na pierwszym lub drugim miejscu
			if (locationMap.containsKey("/" + splitted[0])) {
				location = "/" + splitted[0];
			} else if (splitted.length > 1 && locationMap.containsKey("/" + splitted[1])) {
				location = "/" + splitted[1];
			}
		}
		return location;
	}
	
	public static long findRfpIdInUrl(String url) {
		long idRfp = -1;
		if (url != null && !url.isEmpty()) {
			Matcher match = PATTERN_SHOW_DETAILS.matcher(url);
			if (match.find()) {
				if (match.groupCount() >= 3) {
					String idRfpTmp = match.group(3);
					if (idRfpTmp != null && !idRfpTmp.isEmpty()) {
						idRfp = Long.parseLong(idRfpTmp);
					}
				}
			}
		}
		return idRfp;

	}

	public static String getRfpListHeaderPd(String categoryName, String locationName, String districtName, String defaultTitle)
	{
		StringBuffer result = new StringBuffer(150);
		boolean added = false;
		if (categoryName != null && categoryName.trim().length() > 0) {
			String categoryCapitalize = toUpperFirstLetters(categoryName, " ", false);
			result.append(categoryCapitalize);
			added = true;
		}
		
		if (districtName != null && districtName.trim().length() > 0) {
			String districtTmp = districtName;
			if (districtTmp != null) {
				if (added) {
					result.append(" ");
				}
				result.append(districtName).append(" ");
			}
		} else {
			if (locationName != null && locationName.trim().length() > 0) {
				if (added) {
					result.append(" ");
				} else {
					result.append(defaultTitle).append(" ");
				}
				String locCapialize = toUpperFirstLetters(locationName, " ", false);
				result.append(locCapialize);
				added = true;
			}
		}
		
		if (added == false) {
			result.append(defaultTitle);
		}
		return result.toString().trim();
	}

	private static String toUpperFirstLetters(String inString, String delimeter, boolean onlyfirstWord) 
	{
		return toUpperFirstLetters(inString, delimeter, onlyfirstWord, false);
	}

	private static String toUpperFirstLetters(String inString, String delimeter, boolean onlyfirstWord, boolean capitalizeOneLetter) 
	{
		String result = inString;
		if (inString != null && inString.length() > 0) {
			int idx = 0;
			result = "";
			StringTokenizer st = new StringTokenizer(inString, delimeter);
			while(st.hasMoreTokens())  {
			    String s = st.nextToken();
			    String sTmp = s;
			    if ((onlyfirstWord && idx == 0) || (onlyfirstWord == false)) {
			    	if ((capitalizeOneLetter && s.length() == 1) || (s.length() > 1)) {
				    	if (isCapitalizeWord(s)) {
				    		sTmp = s.substring(0,1).toUpperCase() + s.substring(1).toLowerCase();
				    	}
			    	}
			    }
			    result += sTmp;
			    if (st.hasMoreElements()) {
			    	result += delimeter;
			    }
			    idx++;
			}
		}		
		return result;
	}

	private static boolean isCapitalizeWord(String inString) 
	{
		boolean result = true;
		
		if (inString != null) {
			String word = inString;
			if (inString.endsWith(",") || inString.endsWith(".")) {
				word = inString.substring(0, inString.length() - 1);
			}
			if (CAPITALIZE_EXCLUDED_WORDS.containsKey(word.toUpperCase())) {
				result = false;
			}
		}
		return result;
	}
	
	public static String getShortedText(String text, int length)
	{
		if (text != null) {
			text = text.trim();
			text = trimText(text, ".", null);
			text = trimText(text, ",", null);
			text = trimText(text, ":", null);
			text = trimText(text, ";", null);
			text = trimText(text, ")", null);
			text = trimText(text, "}", null);
			text = trimText(text, "]", null);
			text = trimText(text, "/", null);
			text = trimText(text, "?", null);
			text = trimText(text, "-", null);
			text = text.trim();
			
			if (length >= 0 && text.length() >= length) {
				text = text.substring(0, length);
			}
		}
		return text;
	}

	
	public static String changeSuffix(String inputText, long number, String[] endings)
	{
		String result = ""; 
		if (number == 1) {
			result = inputText + endings[0];
		} else {
			long lastDigit 	= number % 10;
			long mod100 	= number % 100;
			if ((lastDigit > 1 && lastDigit < 5) && !(number > 10 && number < 20) && !(mod100 > 10 && mod100 < 20)) {
				result = inputText + endings[1]; // komentarze
			} else {
				result = inputText + endings[2];  // komentarzy
			}
		}
		return result;
	}
	
	public static String[] CLIENT_SUFFIXES_PL 		= new String[]{ "", "ów",  "ów" };

	public static String formatNumber(long number) 
	{
		NumberFormat nformat = NumberFormat.getNumberInstance();
		String result = nformat.format(number);
		return result;
	}

	public static String replaceNewLinesToHTML(String text)
	{
		if (text == null) return "";
		
		String[] rows = text.split("([ ]*\r\n)|([ ]*\n)");
		String[] elems;
		StringBuffer result = new StringBuffer(1000);
		
		for (int i = 0; i < rows.length; i++) {
			elems = rows[i].split(":", 2);
			
			if (elems.length == 2 && !elems[0].equalsIgnoreCase(HTTP) && !elems[0].equalsIgnoreCase(HTTPS))
				result.append("<b>").append(elems[0]).append("</b>").append(": ").append(elems[1]);
			else
				result.append(rows[i]);
			
			if (i+1 < rows.length)
				result.append("<br/>");
		}
					
		return result.toString();		
	}
	
	public static int getSortFromUrl(String urlQueryString) {
		int sort = OferteoApi.DEFAULT_SORT;
		if (urlQueryString != null && !urlQueryString.isEmpty()) {
			Matcher match = PATTERN_SORT.matcher(urlQueryString);
			if (match.find()) {
				if (match.groupCount() >= 2) {
					String sortTmp = match.group(2);
					if (sortTmp != null && !sortTmp.isEmpty()) {
						sort = Integer.parseInt(sortTmp);
					}
				}
			}
		}
		return sort;
	}
	
	public static String getPagerMetaTags(String currentUrl, long itemsCount, int itemsPerPage) {
		
		int pagerFromUrl = findPagerInUrl(currentUrl);
		if (pagerFromUrl < 0) {
			pagerFromUrl = 1;
		}
		long pages = 1;
		if (itemsCount > itemsPerPage) {
			boolean addOne = itemsCount % itemsPerPage > 0;
			pages = itemsCount / itemsPerPage ;
			if (addOne) {
				pages++;
			}
		}

		StringBuilder sb = new StringBuilder();
		
		boolean pagesExist = itemsCount > itemsPerPage;
		boolean firstPage = (pagerFromUrl == 1);
		boolean lastPage = pagerFromUrl == pages;

		if (pagesExist) {
			if (!lastPage) {
				sb.append("<link rel=\"next\" href=\"" + getNextUrl(currentUrl, pagerFromUrl) + "\"/>");
			}
			if (!firstPage) {
				sb.append("<link rel=\"prev\" href=\"" + getPrevUrl(currentUrl, pagerFromUrl) + "\"/>");
				sb.append("<meta name=\"robots\" content=\"noindex,follow\" />");
			}
		}
		return sb.toString();
	}
	
	private static String getPrevUrl(String url, int pageNbr) {
		String prevUrl = "";
		String pagerStringCurrent 	= ""; 
		String pagerStringPrev	 	= "";
		
		pagerStringCurrent 	= DEFAULT_PARAM_URL_NAME + (pageNbr); 
		pagerStringPrev 	= DEFAULT_PARAM_URL_NAME + (pageNbr > 2 ? (pageNbr - 1) : "");

		
		if (url != null && url.indexOf(pagerStringCurrent) >= 0) {
			prevUrl = url.replace(pagerStringCurrent, pagerStringPrev);
		} else {
			prevUrl = url + "/";
		}

		return prevUrl;
	}
	
	
	private static String getNextUrl(String url, int pageNbr) {
		
		String nextUrl 				= "";
		String pagerStringCurrent 	= ""; 
		String pagerStringNext	 	= "";
		
		pagerStringCurrent 	= DEFAULT_PARAM_URL_NAME + (pageNbr); 
		pagerStringNext 	= DEFAULT_PARAM_URL_NAME + (pageNbr + 1);
		
		if (url != null && url.indexOf(pagerStringCurrent) >= 0) {
			nextUrl = url.replace(pagerStringCurrent, pagerStringNext);
		} else {
			nextUrl = url + "/" + (pageNbr + 1);
		}
		return nextUrl;
	}


}
