package com.oferteo.api.v1.client.rest;

import java.util.Collection;
import java.util.Locale;

public class CategoriesSingleColumnHtmlBuilder extends CategoriesHtmlBuilder
{
	private static final int VIRTUAL_ALL_CATEGORY = 9999;
	private static final String ALL_CATEGORY = "Wszystkie";
	private Locale locale;
	private boolean showingRfp;


	public CategoriesSingleColumnHtmlBuilder(Collection<CategoryRest> categoriesTree, String urlPrefix, String locationSelected, Locale locale, boolean showingRfp) {
		super(categoriesTree, urlPrefix, locationSelected);
		this.locale = locale;
		this.showingRfp = showingRfp;
	}
	
	public String getHtml() 
	{
		
		Collection<CategoryRest> categoriesTop = getCategoriesTree();
		CategoryRest selectedCategory = getSelectedCategory(categoriesTop);

		if (selectedCategory != null) {
			if (showingRfp) {
				getSb().append("<h2 class=\"title-inline\">").append(ApiClientUtils.getMessage(locale, "rfp.search.search.more.category")).append("</h2>");
			} else {
				getSb().append("<h2 class=\"title-inline\">").append(ApiClientUtils.getMessage(locale, "rfp.search.choose.category")).append("</h2>");
			}
			openUl("");
			addBreadCrumbAll();
			
			addCategories(categoriesTop);
			closeUl();
		} else {
			getSb().append("<h2>").append(ApiClientUtils.getMessage(locale, "rfp.search.choose.category")).append("</h2>");
			addCategories(categoriesTop);
		}
		
		return getSb().toString();
	}
	
	private void addBreadCrumbAll()
	{
		openLi("", "http://data-vocabulary.org/Breadcrumb");
	
		String url = getUrlPrefix() + (getLocationSelected() != null ? getLocationSelected() : "");
		openAnchor("", null, url);
		getSb().append(ALL_CATEGORY);
		closeAnchor();
		
		closeLi();//bcrumb		
	}
	
	private void addBreadCrumbRaquo() {
		getSb().append("<li class=\"raquo\"><span>&raquo;</span></li>");
	}

	private void addBreadCrumb(CategoryRest category, boolean selected)
	{
		addBreadCrumbRaquo();

		openLi("", "http://data-vocabulary.org/Breadcrumb");

		String cssClass = "";
		if (selected) {
			cssClass = "a-selected";
		}
		openAnchor(cssClass, null, getCategoryUrl(category));
		getSb().append(category.getName());
		closeAnchor();

		closeLi();//bcrumb
	}

	private CategoryRest getSelectedCategory(Collection<CategoryRest> categoriesTop)
	{
		for (CategoryRest category : categoriesTop) {
			if (category.isSelected() && category.getId() != 9999) {
				return category;
			}
		}
		return null;
	}


	protected void addCategories(Collection<CategoryRest> categories) 
	{
		CategoryRest selectedCategory = getSelectedCategory(categories);
		if (selectedCategory != null) {
			//dorysuj breadCrumb do wybranego elementu
			boolean hasChildSelected = (getSelectedCategory(selectedCategory.getSubcategories()) != null);
			boolean currentMarkSelected = !hasChildSelected;

			addBreadCrumb(selectedCategory, currentMarkSelected);

			//rysuj kategorie ponizej zaznaczonego elementu
			if (!selectedCategory.getSubcategories().isEmpty()) {
				addCategories(selectedCategory.getSubcategories());
			}
			
		} else {
			//rysuj pelna liste, ale bez potomnych
			openUl("sub-categories");
			for (CategoryRest category : categories) {
				if (category.getId() == VIRTUAL_ALL_CATEGORY) {
					continue;
				}
				openCatLocLi(category.isSelected(), category.isLeaf());
					StringBuilder sb = getSb();
					
					//category anchor
					String url = getCategoryUrl(category);
					String cssClass = (category.isSelected() ? "a-selected" : null);
					openAnchor(cssClass, null, url, "idCat");
						sb.append(category.getName());
					closeAnchor();
				closeLi();
			}
			closeUl();
		}
	}


}
