package com.oferteo.api.v1.client.rest;

import java.util.Collection;

public class LocationsHtmlBuilder extends HtmlBuilder 
{
	
	private Collection<LocationRest> locationsTree;
	private String urlPrefix;
	private boolean noSelection;
	private String categorySelected;


	/**
	 * @param locationsTree - locations
	 * @param urlPrefix - prefix added to each url
	 * @param noSelection - true if no location is selected
	 */
	public LocationsHtmlBuilder(Collection<LocationRest> locationsTree, String urlPrefix, String categorySelected) 
	{
		this.locationsTree = locationsTree;
		this.urlPrefix = urlPrefix;
		this.noSelection = !isAnyLocationSelected(locationsTree);
		this.categorySelected = categorySelected;
	}
	
	private boolean isAnyLocationSelected(Collection<LocationRest> locations) 
	{
		for (LocationRest location : locations) {
			if (location.isSelected()) {
				return true;
			}
		}
		return false;
	}
	
	
	public String getHtml() 
	{
		
		addLocations(locationsTree);
		
		return getSb().toString();
	}


	
	private void addLocations(Collection<LocationRest> locations) {
		
		boolean noLocationSelectedOnTreeLevel = !isAnyLocationSelected(locations);

		openUl();
		//jesli zaden nie wybrany to rysujemy wszystkie
		boolean last = false;
		int index = 0;
		for (LocationRest location : locations) {
			index++;
			last = (index == locations.size());
			boolean draw = location.isSelected() || noLocationSelectedOnTreeLevel || location.getParent() == null || location.isLeaf();
			if (draw) {
				
				if (location.isLeaf()) {
					openCatLocLi(location.isSelected(), location.isLeaf());
					StringBuilder sb = getSb();
					
					String url = getLocationUrl(location);
					String cssClass = location.isSelected() ? "a-selected" : null;
					openAnchor(cssClass, null, url, "idLoc");
					sb.append(location.getName());
					if (!last) {
						sb.append(", ");
					}
					closeAnchor();
					closeLi();
					
				} else {
					
					openCatLocLi(location.isSelected(), location.isLeaf());
					StringBuilder sb = getSb();
					
					String url = getLocationUrl(location);
					String cssClass = location.isSelected() ? "a-selected" : null;
					openAnchor(cssClass, null, url, "idLoc");
					sb.append(location.getName());
					closeAnchor();
	
					//open when selected and has children
					//or when nothing was selected and locationPL (level < 0)
					if (!location.getLocations().isEmpty() && location.isSelected()
							|| (location.getLevel() < 0 && this.noSelection)) {
						addLocations(location.getLocations());
					}
					closeLi();
				}
			}
		}
		closeUl();
		
	}

	protected String getLocationUrl(LocationRest location)
	{
		String url = this.urlPrefix
				+ (categorySelected != null ? categorySelected : "")
				+ location.getUrlLocation()
				;
		return url;
	}

	protected Collection<LocationRest> getLocationsTree()
	{
		return locationsTree;
	}

	protected String getUrlPrefix()
	{
		return urlPrefix;
	}

	protected boolean isNoSelection()
	{
		return noSelection;
	}

	protected String getCategorySelected()
	{
		return categorySelected;
	}
}
