package com.oferteo.api.v1.client.rest;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;

public class LocationsSingleColumnHtmlBuilder extends LocationsHtmlBuilder 
{
	
	private static final Object ALL_LOCATION = "Wszystkie";
	private String selectedPoviatName;
	private String selectedPoviatUrl;
	private boolean showingRfp;
	private Locale locale;
	private String selectedCityName;

	/**
	 * @param locationsTree - locations
	 * @param urlPrefix - prefix added to each url
	 * @param noSelection - true if no location is selected
	 */
	public LocationsSingleColumnHtmlBuilder(Collection<LocationRest> locationsTree, String urlPrefix, String categorySelected, boolean showingRfp, String selectedCityName, String selectedPoviatName, String selectedPoviatUrl, Locale locale) 
	{
		super(locationsTree, urlPrefix, categorySelected);
		this.selectedCityName = selectedCityName;
		this.selectedPoviatName = selectedPoviatName;
		this.selectedPoviatUrl = selectedPoviatUrl;
		this.showingRfp = showingRfp;
		this.locale = locale;
	}

	private LocationRest getLocationSelected(Collection<LocationRest> locations) 
	{
		for (LocationRest location : locations) {
			if (location.isSelected()) {
				return location;
			}
		}
		return null;
	}
	
	
	private void addBreadCrumbAll(boolean selected)
	{
		openLi("", "http://data-vocabulary.org/Breadcrumb");
	
		String url = getUrlPrefix() + (getCategorySelected() != null ? getCategorySelected() : "");
		String cssClass = "";
		if (selected) {
			cssClass = "a-selected";
		}
		openAnchor(cssClass, null, url);
		getSb().append(ALL_LOCATION);
		closeAnchor();
		
		closeLi();//bcrumb		
	}
	
	private void addBreadCrumbRaquo() {
		getSb().append("<li class=\"raquo\"><span>&raquo;</span></li>");
	}
	
	private void addBreadCrumb(LocationRest location, boolean selected)
	{
		addBreadCrumbRaquo();

		openLi("", "http://data-vocabulary.org/Breadcrumb");

		String cssClass = "";
		if (selected) {
			cssClass = "a-selected";
		}
		openAnchor(cssClass, null, getLocationUrl(location));
		getSb().append(location.getName());
		closeAnchor();

		closeLi();//bcrumb
	}


	
	public String getHtml() 
	{
		Collection<LocationRest> locationsTop = getLocationsTree();
		LocationRest locationSelected = getLocationSelected(locationsTop);
		
		if (locationSelected != null) {
			if (showingRfp) {
				getSb().append("<h2 class=\"title-inline\">").append(ApiClientUtils.getMessage(locale, "rfp.search.search.more.location")).append("</h2>");
			} else {
				getSb().append("<h2 class=\"title-inline\">").append(ApiClientUtils.getMessage(locale, "rfp.search.choose.location")).append("</h2>");
			}
			openUl("");
			addBreadCrumbAll(false);
			
			addLocations(locationsTop);
			closeUl();
			
		} else {
			
			getSb().append("<h2 class=\"title-inline\">").append(ApiClientUtils.getMessage(locale, "rfp.search.choose.location")).append("</h2>");
			//domyslnie rozwinieta polska
			//dodaj element 'wszystkie'
			openUl("");
			addBreadCrumbAll(true);
			LocationRest locationPl = findLocation(locationsTop, "locationPl");
			
			//dodaj polske
			addBreadCrumb(locationPl, false);

			//dodaj wojewodztwa w polsce
			ArrayList<LocationRest> locationsPlAndForeign = new ArrayList<LocationRest>(locationPl.getLocations());
			locationsPlAndForeign.add(findLocation(locationsTop, "locationAbroad"));
			addLocations(locationsPlAndForeign);
			closeUl();//locations
		}
		return getSb().toString();
	}

	private LocationRest findLocation(Collection<LocationRest> locations, String locationId)
	{
		if (locationId == null || locations == null) {
			return null;
		}
		for (LocationRest location : locations) {
			if (locationId.equals(location.getId())) {
				return location;
			}
		}
		return null;
	}

	private void addLocations(Collection<LocationRest> locations) {
		
		LocationRest locationSelected = getLocationSelected(locations);
		if (locationSelected != null) {
			//dorysuj breadCrumb do wybranego elementu
			boolean hasChildSelected = (getLocationSelected(locationSelected.getLocations()) != null);
			
			boolean currentMarkSelected = !hasChildSelected && !showingRfp;
			
			addBreadCrumb(locationSelected, currentMarkSelected);

			//rysuj kategorie ponizej zaznaczonego elementu
			if (!locationSelected.getLocations().isEmpty()) {
				addLocations(locationSelected.getLocations());
			}
			
		} else {
			if (showingRfp) {
				//w trybie wyswietlania jednego RFP pokazujemy:
				// - powiat z linkiem jesli sa do tego dane
				// - miasto jako napis, jesli nie ma danych powiatu
				if (this.selectedPoviatUrl != null && this.selectedPoviatUrl.length() > 0) {
					addBreadCrumbRaquo();

					openLi("", "http://data-vocabulary.org/Breadcrumb");

						String url = this.getUrlPrefix()
								+ (getCategorySelected() != null ? getCategorySelected() : "")
								+ this.selectedPoviatUrl
								;
						openAnchor("a-selected", this.selectedPoviatName, url, "idLoc");
							getSb().append(this.selectedPoviatName);
						closeAnchor();
					closeLi();//bcrumb		
				} else {
					if (this.selectedCityName != null && this.selectedCityName.length() > 0) {
						addBreadCrumbRaquo();
						//miasto bez linka
						openLi("", "http://data-vocabulary.org/Breadcrumb");
							getSb().append(this.selectedCityName);
						closeLi();//bcrumb		
					}
				}
			} else {
				//rysuj pelna liste, ale bez potomnych
				openUl("sub-locations");
					for (LocationRest location : locations) {
						openCatLocLi(location.isSelected(), location.isLeaf());
							StringBuilder sb = getSb();
							
							//category anchor
							String url = getLocationUrl(location);
							String cssClass = (location.isSelected() ? "a-selected" : null);
							openAnchor(cssClass, null, url, "idLoc");
								sb.append(location.getName());
							closeAnchor();
						closeLi();
					}
				closeUl();
			}
		}		
	}
}
