package com.oferteo.api.v1.client.rest;

import java.util.Locale;

public class RfpsHtmlBuilder extends HtmlBuilder 
{

	private RfpSearchResult rfpSearch;
	private String urlPrefix;
	private Locale locale;
	private String currentUrl;
	private String selectedCategoryUrl;
	private String selectedLocationUrl;
	private int pageSize;
	private int sort;
	private String urlQueryString;
	private String widgetId;
	
	public RfpsHtmlBuilder(RfpSearchResult rfpSearch, String urlPrefix, Locale locale, String currentUrl, String selectedCategoryUrl, String selectedLocationUrl, int pageSize, int sort, String urlQueryString, String widgetId) 
	{
		super();
		this.rfpSearch = rfpSearch;
		this.urlPrefix = urlPrefix;
		this.locale = locale;
		this.currentUrl = currentUrl;
		this.selectedCategoryUrl = selectedCategoryUrl;
		this.selectedLocationUrl = selectedLocationUrl;
		this.pageSize = pageSize;
		this.sort = sort;
		this.urlQueryString = urlQueryString;
		this.widgetId = widgetId;
	}
	
	public String getHtml() 
	{
		getSb().setLength(0);
		
		getHeaderHtml();
		MainCitiesHtmlBuilder mainCitiesBuilder = new MainCitiesHtmlBuilder(this.selectedCategoryUrl, this.urlPrefix, false, getLocale());
		getSb().append(mainCitiesBuilder.getHtml());
		
		addRfpListAndPagination();
		
		return getSb().toString();
	}

	public String getHeaderHtml()
	{
		getSb().setLength(0);

		getSb().append("<h1>");
		getRfpsListTitle(ApiClientUtils.getMessage(locale, "rfp.search.title.default")); 
		getSb().append("</h1>");
		
		openAnchor("powered-by", ApiClientUtils.getMessage(locale, "rfp.search.poweredby"), "http://www.oferteo.pl");
			getSb().append("<img src=\"http://www.oferteo.pl/images/forms/dzial-zlecen/powered-by.png\" alt=\"").append(ApiClientUtils.getMessage(locale, "rfp.search.poweredby")).append("\" />");
		closeAnchor();
		addDivClear();
		
		getAskForOfferHtml();
		return getSb().toString();
	}

	protected void addRfpListAndPagination()
	{
		if (rfpSearch.getRfpsCounter() > 0) {
			addSummary();
		
			int i = 0;
			for (RfpRest rfp : rfpSearch.getRfps()) {
				addRfpElement(rfp, i++);
			}
			getPaginationAndSortHtml();
		} else {
			openDiv("not-found-info");
			openSpan();
			getSb().append(ApiClientUtils.getMessage(locale, "rfp.search.results.nodata"));
			closeSpan();
			closeDiv();//not-found-info
		}
	}

	/**
	 * Render <div> element with description of one RFP
	 * @param rfp
	 */
	private void addRfpElement(RfpRest rfp, int i) 
	{
		String classGrey = i % 2 == 0 ? "" : " background-grey";
		openDiv("item" + classGrey);
		
		//left side
		openDiv("item-panel-left");
		addLeftRfpHtml(rfp);
		closeDiv();//left
		
		//right side
		openDiv("item-panel-right");
		addRightRfpHtml(rfp);
		closeDiv();//right
		addDivClear();
		
		closeDiv();//item
	}

	private void addRightRfpHtml(RfpRest rfp) 
	{
		
		openDiv("date-item");
		openSpan();
		if (this.sort == OferteoApi.DEFAULT_SORT) {
			getSb().append(rfp.getActivationTimeStr()).append(ApiClientUtils.getMessage(locale, "rfp.search.rfp.fromTo")).append(rfp.getExpirationTimeStr());
		} else {
			getSb().append(rfp.getActivationTimeStr()).append(ApiClientUtils.getMessage(locale, "rfp.search.rfp.fromTo")).append(rfp.getExpirationTimeStr());
		}
		closeSpan();

		if (rfp.isActive() == false) {
			getSb().append("<br/>");
			openSpan(rfp.getExpirationPeriodStyle());
			getSb().append(rfp.getExpirationPeriod());
			closeSpan();
		} else {
			if (this.sort == OferteoApi.DEFAULT_SORT) {
				getSb().append(ApiClientUtils.getMessage(locale, "rfp.details.date.added")).append(rfp.getActivationPeriod());
			} else {
				getSb().append(ApiClientUtils.getMessage(locale, "rfp.details.date.valid.to"));
				openSpan(rfp.getExpirationPeriodStyle());
				getSb().append(rfp.getExpirationPeriod());
				closeSpan();
			}
		} 
		
		closeDiv(); //date-item
		
		addDivClear();
		
	}

	private void addLeftRfpHtml(RfpRest rfp) 
	{
		addRfpIconDiv(rfp);
		addTitleAnchor(rfp);
		addDivClear();
		addDescription(rfp);
		addCategoryAndLocation(rfp);
	}

	private void addCategoryAndLocation(RfpRest rfp) 
	{
		openDiv("category-item");
		
		openDiv("tags-item");
		String categoryUrl = urlPrefix
				 + rfp.getCategoryUrl()
				 + (this.selectedLocationUrl != null ? this.selectedLocationUrl : "");
		openAnchor("link", rfp.getCategoryName(), categoryUrl);
		getSb().append(rfp.getCategoryName());
		closeAnchor();
		closeDiv();
		
		openDiv("location-item");
		if (rfp.getDstVoivodship() != null && rfp.getDstVoivodship().length() > 0) {
			
			String url = urlPrefix 
					+ (this.selectedCategoryUrl != null ? this.selectedCategoryUrl : "")
					+ rfp.getDstVoivodshipUrl();
			String title = (rfpSearch.getSelectedCategoryName() != null ? rfpSearch.getSelectedCategoryName() : "") + " " + rfp.getDstVoivodship();
			openAnchor("link", title, url);
			getSb().append(rfp.getDstVoivodship());
			closeAnchor();
			if (rfp.getDstCity() != null && rfp.getDstCity().length() > 0) {
				getSb().append(" / ");
			}
		}
		getSb().append(rfp.getDstCity());
		closeDiv();
		addDivClear();
		
		closeDiv();
	}

	/**
	 * Opis z rfp
	 * @param rfp
	 */
	private void addDescription(RfpRest rfp) 
	{
		openDiv("text-item");
		getSb().append(ApiClientUtils.getShortedText(rfp.getDescription(), 230)).append("...");
		closeDiv();
	}

	private void addTitleAnchor(RfpRest rfp) 
	{
		openAnchor("title-item", rfp.getTitle(), urlPrefix + "/" + rfp.getRfpUrl());
		getSb().append(rfp.getTitle());
		closeAnchor();
		
	}

	protected void addRfpIconDiv(RfpRest rfp) 
	{
	}

	/**
	 * Dodaje div z informacja podsumowujaca liste wyszukanych elementow
	 */
	private void addSummary() 
	{
		openDiv("information-box background-grey");
		
		openDiv("customers-number");
		getSb().append("<h2>");
		getRfpsCustomersNumber();
		getSb().append("</h2>");
		closeDiv();//customers-number
		
		addDivClear();
		
		closeDiv();//information-box
	}
	
	protected void getRfpsListTitle(String defaultMessage) {
		getSb().append(ApiClientUtils.getRfpListHeaderPd(rfpSearch.getSelectedCategoryName(), rfpSearch.getSelectedLocationName(), null, defaultMessage));
	}
	
	private void getRfpsCustomersNumber() {
		getSb().append(ApiClientUtils.formatNumber(rfpSearch.getRfpsCounter()));
		getSb().append("&nbsp;").append(ApiClientUtils.changeSuffix("Klient", rfpSearch.getRfpsCounter(), ApiClientUtils.CLIENT_SUFFIXES_PL));
		getSb().append("&nbsp;na&nbsp;");
		getRfpsListTitle("Usługi i Towary");
	}

	protected void getPaginationAndSortHtml() {
		int pagerFromUrl = ApiClientUtils.findPagerInUrl(this.currentUrl);
		if (pagerFromUrl < 0) {
			pagerFromUrl = 1;
		}
		long pages = 1;
		if (rfpSearch.getRfpsCounter() > pageSize) {
			boolean addOne = rfpSearch.getRfpsCounter() % pageSize > 0;
			pages = this.rfpSearch.getRfpsCounter() / pageSize ;
			if (addOne) {
				pages++;
			}
		}
		
		openDiv("pagination-sort-box");
		
		addPagination(pages, pagerFromUrl);

		closeDiv();//pagination-sort-box
	}

	private void addSortBox() 
	{
		openDiv("sort-box");
		getSb().append("<strong>").append(ApiClientUtils.getMessage(locale, "rfp.search.results.sort")).append("</strong>");
		
		if (this.sort == OferteoApi.DEFAULT_SORT) {
			openSpan();
			getSb().append(ApiClientUtils.getMessage(locale, "rfp.search.results.sort.activation"));
			closeSpan();
			getSb().append("<span class=\"separator\">&nbsp;|&nbsp;</span>");
			openAnchor(null, null, getUrlCurrentSelection() + "?sort=1");
			getSb().append(ApiClientUtils.getMessage(locale, "rfp.search.results.sort.expiry"));
			closeAnchor();
		} else {
			openAnchor(null, null, getUrlCurrentSelection() + "");
			getSb().append(ApiClientUtils.getMessage(locale, "rfp.search.results.sort.activation"));
			closeAnchor();
			getSb().append("<span class=\"separator\">&nbsp;|&nbsp;</span>");
			openSpan();
			getSb().append(ApiClientUtils.getMessage(locale, "rfp.search.results.sort.expiry"));
			closeSpan();
		}

		closeDiv();//sort-box
	}

	private String getUrlCurrentSelection() 
	{
		String category = this.selectedCategoryUrl != null ? this.selectedCategoryUrl : "";
		String location = this.selectedLocationUrl != null ? this.selectedLocationUrl : "";
		return this.urlPrefix + category + location;
	}
	
	private void addPagination(long pages, long currentPage) 
	{
		openDiv("pagination-item");

		int sort = ApiClientUtils.getSortFromUrl(urlQueryString);
		String sortParam ="";
		if (sort != OferteoApi.DEFAULT_SORT) {
			sortParam = "?sort=" + sort;
		}
		
		//prev arrows
		long prevPage = currentPage - 1;
		if (prevPage > 0) {
			openAnchor("arrow-all-next", null, getUrlCurrentSelection() + sortParam);	//first page
			closeAnchor();
			if (prevPage == 1) {
				//pierwsza strona bez numerka
				openAnchor("arrow-next", null, getUrlCurrentSelection() + sortParam);	//prevpage
				closeAnchor();
			} else {
				openAnchor("arrow-next", null, getUrlCurrentSelection() + "/" + prevPage + sortParam);	//prevpage
				closeAnchor();
			}
			
		}
		
		//visible pages
		long maxPages = 10;
		long firstVisiblePage = 1;
		long lastVisiblePage = pages;
		if (pages > maxPages) {
			firstVisiblePage = Math.max(1, currentPage - (maxPages/2));
			lastVisiblePage = Math.min(pages, firstVisiblePage + maxPages);
		}
		for (long i = firstVisiblePage; i <= lastVisiblePage; i++) {
			if (i == currentPage) {
				openSpan();
				getSb().append(i);
				closeSpan();
			} else {
				if (i == 1) {
					openAnchor(null, null, getUrlCurrentSelection() + sortParam);
					getSb().append(i);
					closeAnchor();
				} else {
					openAnchor(null, null, getUrlCurrentSelection() + "/" + i + sortParam);
					getSb().append(i);
					closeAnchor();
				}
			}
		}
		
		//next arrows
		long nextPage = currentPage + 1;
		if (nextPage <= pages) {
			openAnchor("arrow-all-back", null, getUrlCurrentSelection() + "/" + nextPage + sortParam);	//prevpage
			closeAnchor();
			openAnchor("arrow-back", null, getUrlCurrentSelection() + "/" + pages + sortParam);	//last page
			closeAnchor();
		}
		
		openDiv("page-number");
		getSb().append(ApiClientUtils.getMessage(locale, "rfp.search.results.currentPage", currentPage, pages));
		closeDiv();//page-number
		
		closeDiv();//pagination-item
	}
	
	protected String getAskForOfferHtml() 
	{
		openDiv("cta-box");
		if (widgetId != null && !widgetId.equals("")) {
			getSb().append("<style type='text/css'> .w-2 { width:100%; height:74px; } </style>");
			getSb().append("<div class='w-2 ").append(widgetId).append("'></div>");
		} else {
			String categoryUrl = "";
			if (this.selectedCategoryUrl != null) {
				categoryUrl = this.selectedCategoryUrl + "/zapytanie";
			} else {
				categoryUrl = "/zapytaj-o-oferte";
			}
			String ocUrl = "";
			if (this.rfpSearch.getOc() != null) {
				ocUrl = "?oc=PP_" + this.rfpSearch.getOc();
			}
			String url = this.rfpSearch.getOferteoPrefixUrl() + categoryUrl + ocUrl;
			openAnchor("btn btn-success", ApiClientUtils.getMessage(locale, "rfp.search.newRfp"), url );
			getSb().append("<strong>").append(ApiClientUtils.getMessage(locale, "rfp.search.newRfp")).append("</strong>");
			getSb().append("<br/><span>").append(ApiClientUtils.getMessage(locale, "rfp.search.newRfp.free")).append("</span>");
			closeAnchor();
			
			openUl();
			openLi();
			getSb().append(ApiClientUtils.getMessage(locale, "rfp.search.newRfp.part1"));
			closeLi();
			openLi();
			getSb().append(ApiClientUtils.getMessage(locale, "rfp.search.newRfp.part2"));
			closeLi();
			closeUl();
			
			addDivClear();
		}
		closeDiv();//cta-box
		
		return getSb().toString();
	}

	protected RfpSearchResult getRfpSearch()
	{
		return rfpSearch;
	}

	protected String getUrlPrefix()
	{
		return urlPrefix;
	}

	protected Locale getLocale()
	{
		return locale;
	}

	protected String getCurrentUrl()
	{
		return currentUrl;
	}

	protected String getSelectedCategoryUrl()
	{
		return selectedCategoryUrl;
	}

	protected String getSelectedLocationUrl()
	{
		return selectedLocationUrl;
	}

	protected int getPageSize()
	{
		return pageSize;
	}

	protected int getSort()
	{
		return sort;
	}

	protected String getUrlQueryString()
	{
		return urlQueryString;
	}
	
}
